//
//  RMDocument.m
//  RaiseMan
//
//  Created by narita on 2013/04/05.
//  Copyright (c) 2013年 narita. All rights reserved.
//

#import "RMDocument.h"
#import "Person.h"
#import "PreferenceController.h"

static void* RMDocumentKVOContext;


@implementation RMDocument

- (id)init
{
    self = [super init];
    
    if (self)
    {
        employees = [[NSMutableArray alloc] init];
    }
    
    return self;
}




- (void) startObservingPerson:(Person*)inPerson
{
    [inPerson addObserver:self
               forKeyPath:@"personName"
                  options:NSKeyValueObservingOptionOld
                  context:&RMDocumentKVOContext];
    
    [inPerson addObserver:self
               forKeyPath:@"expectedRaise"
                  options:NSKeyValueObservingOptionOld
                  context:&RMDocumentKVOContext];
}



- (void) stopObservingPerson:(Person*)inPerson
{
    [inPerson removeObserver:self
                  forKeyPath:@"personName"
                     context:&RMDocumentKVOContext];
    
    [inPerson removeObserver:self
                  forKeyPath:@"expectedRaise"
                     context:&RMDocumentKVOContext];
}



- (IBAction) createEmployee:(id)sender
{
    NSWindow* theWindow = [tableView window];

    
    BOOL theEditingEnded = [theWindow makeFirstResponder:theWindow];
    if( !theEditingEnded )
    {
        return;
    }
    
    NSUndoManager* theUndoMgr  = [self undoManager];
    
    if( [theUndoMgr groupingLevel] > 0 )
    {
        [theUndoMgr endUndoGrouping];
        
        [theUndoMgr beginUndoGrouping];
    }
    
    Person* thePerson = [employeeController newObject];
    [employeeController addObject:thePerson];
    
    [employeeController rearrangeObjects];
    
    NSArray* theArray = [employeeController arrangedObjects];
    
    NSUInteger theRow = [theArray indexOfObject:thePerson];
    
    [tableView editColumn:0
                      row:theRow
                withEvent:nil
                   select:YES];
}






- (void) setEmployees:(NSMutableArray *)a
{
    for(Person* i in employees)
    {
        [self stopObservingPerson:i];
    }
    
    employees = a;
    
    for(Person* i in employees)
    {
        [self startObservingPerson:i];
    }
}

- (void) changeKeyPath:(NSString*)keyPath
              ofObject:(id)obj
               toValue:(id)newValue
{
    [obj setValue:newValue forKey:keyPath];
}

- (void) observeValueForKeyPath:(NSString *)keyPath
                       ofObject:(id)object
                         change:(NSDictionary *)change
                        context:(void *)context
{
    if( context != &RMDocumentKVOContext )
    {
        [super observeValueForKeyPath:keyPath
                             ofObject:object
                               change:change
                              context:context];
        
        return;
    }
    
    NSUndoManager* theUndoMgr  = [self undoManager];
    RMDocument*    theInstance = [theUndoMgr prepareWithInvocationTarget:self];
    id             theOldValue = [change objectForKey:NSKeyValueChangeOldKey];
    
    if( theOldValue == [NSNull null] )
    {
        theOldValue = nil;
    }
    
    [theInstance changeKeyPath:keyPath
                      ofObject:object
                       toValue:theOldValue];
    
    [theUndoMgr setActionName:@"Edit"];
    
}


- (void) insertObject:(Person*)inPerson inEmployeesAtIndex:(NSUInteger)inIndex
{
    NSUndoManager* theUndoMgr  = [self undoManager];
    RMDocument*    theInstance = [theUndoMgr prepareWithInvocationTarget:self];
    
    // theInstanceをRMDocumentの型にする事で凡ミスが減る
    [theInstance removeObjectFromEmployeesAtIndex:inIndex];
    
    if( ![theUndoMgr isUndoing] )
    {
        [theUndoMgr setActionName:@"Add Person"];
    }
    
    [self startObservingPerson:inPerson];
    
    [employees insertObject:inPerson atIndex:inIndex];
}

- (void) removeObjectFromEmployeesAtIndex:(NSUInteger)inIndex
{
    NSUndoManager* theUndoMgr  = [self undoManager];
    RMDocument*    theInstance = [theUndoMgr prepareWithInvocationTarget:self];
    Person*        thePerson   = [employees objectAtIndex:inIndex];
    
    // theInstanceをRMDocumentの型にする事で凡ミスが減る
    [theInstance insertObject:thePerson
           inEmployeesAtIndex:inIndex];
    
    if( ![theUndoMgr isUndoing] )
    {
        [theUndoMgr setActionName:@"Remove Person"];
    }
    
    [self stopObservingPerson:thePerson];
    
    [employees removeObjectAtIndex:inIndex];
}

- (NSString *)windowNibName
{
    // Override returning the nib file name of the document
    // If you need to use a subclass of NSWindowController or if your document supports multiple NSWindowControllers, you should remove this method and override -makeWindowControllers instead.
    return @"RMDocument";
}

- (void)windowControllerDidLoadNib:(NSWindowController *)aController
{
    [super windowControllerDidLoadNib:aController];

    [tableView setBackgroundColor:[PreferenceController preferenceTableBgColor]];
}

+ (BOOL)autosavesInPlace
{
    return YES;
}

- (NSData *)dataOfType:(NSString *)typeName error:(NSError **)outError
{
    [[tableView window] endEditingFor:nil];
    
    return [NSKeyedArchiver archivedDataWithRootObject:employees];
}

- (BOOL)readFromData:(NSData *)data ofType:(NSString *)typeName error:(NSError **)outError
{
    NSMutableArray* theNewArray = nil;
    
    @try
    {
        theNewArray = [NSKeyedUnarchiver unarchiveObjectWithData:data];
    }
    @catch (NSException *exception)
    {
        if(outError)
        {
            NSDictionary* theDict = [NSDictionary dictionaryWithObject:@"the data is corrupted."
                                                                forKey:NSLocalizedFailureReasonErrorKey];
            
            *outError = [NSError errorWithDomain:NSOSStatusErrorDomain
                                            code:unimpErr userInfo:theDict];
        }
        
        return NO;
    }
    @finally
    {

    }
    
    [self setEmployees:theNewArray];
    
    return YES;
}

@end
